@@ -122,7 +122,9 @@ FCITX_CONFIGURATION(
                                     _("Margin around text")};
     Option<ActionImageConfig> prev{this, "PrevPage", _("Prev Page Button")};
     Option<ActionImageConfig> next{this, "NextPage", _("Next Page Button")};
-    Option<MarginConfig> blurMargin{this, "BlurMargin", _("Blur Margin")};);
+    Option<MarginConfig> blurMargin{this, "BlurMargin", _("Blur Margin")};
+    Option<MarginConfig> shadowMargin{this, "ShadowMargin",
+                                      _("Shadow Margin")};);
 FCITX_CONFIGURATION(
     MenuThemeConfig,
     Option<Color> normalColor{this, "NormalColor", _("Normal text color"),
